@@ -4,6 +4,7 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
+
 #include <config.h>
 
 #ident "$Id$"
@@ -11,21 +12,23 @@
 #include "prototypes.h"
 #include "defines.h"
 
-int get_uid (const char *uidstr, uid_t *uid)
+
+int
+get_uid(const char *uidstr, uid_t *uid)
 {
-	long long int val;
+	long long  val;
 	char *endptr;
 
 	errno = 0;
-	val = strtoll (uidstr, &endptr, 10);
+	val = strtoll(uidstr, &endptr, 10);
 	if (   ('\0' == *uidstr)
 	    || ('\0' != *endptr)
-	    || (ERANGE == errno)
+	    || (0 != errno)
 	    || (/*@+longintegral@*/val != (uid_t)val)/*@=longintegral@*/) {
-		return 0;
+		return -1;
 	}
 
-	*uid = (uid_t)val;
-	return 1;
+	*uid = val;
+	return 0;
 }
 
