@@ -4,33 +4,33 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
+
 #include <config.h>
 
 #ident "$Id$"
 
 #include <stdlib.h>
 #include <errno.h>
+
 #include "prototypes.h"
 
+
 /*
  * getlong - extract a long integer provided by the numstr string in *result
  *
  * It supports decimal, hexadecimal or octal representations.
- *
- * Returns 0 on failure, 1 on success.
  */
-int getlong (const char *numstr, /*@out@*/long int *result)
+int
+getlong(const char *restrict numstr, long *restrict result)
 {
-	long val;
-	char *endptr;
+	char  *endptr;
+	long  val;
 
 	errno = 0;
-	val = strtol (numstr, &endptr, 0);
-	if (('\0' == *numstr) || ('\0' != *endptr) || (ERANGE == errno)) {
-		return 0;
-	}
+	val = strtol(numstr, &endptr, 0);
+	if (('\0' == *numstr) || ('\0' != *endptr) || (0 != errno))
+		return -1;
 
 	*result = val;
-	return 1;
+	return 0;
 }
-
