@@ -16,6 +16,7 @@
 
 #include "alloc.h"
 #include "defines.h"
+#include "memzero.h"
 #include "prototypes.h"
 #include "pwio.h"
 
@@ -69,7 +70,8 @@
 	return pw;
 }
 
-void pw_free (/*@out@*/ /*@only@*/struct passwd *pwent)
+void
+pw_free(/*@only@*/struct passwd *pwent)
 {
 	if (pwent != NULL) {
 		free (pwent->pw_name);
