@@ -12,32 +12,35 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+
+#include "atoi/strtou_noneg.h"
 #include "defines.h"
 #include "prototypes.h"
 #include "subordinateio.h"
 #include "idmapping.h"
+#include "shadowlog.h"
 
-const char *Prog;
-FILE *shadow_logfd = NULL;
+static const char Prog[] = "check_subid_range";
 
 int main(int argc, char **argv)
 {
 	char *owner;
 	unsigned long start, count;
 	bool check_uids;
-	Prog = Basename (argv[0]);
-	shadow_logfd = stderr;
+	log_set_progname(Prog);
+	log_set_logfd(stderr);
 
 	if (argc != 5)
 		exit(1);
 
 	owner = argv[1];
 	check_uids = argv[2][0] == 'u';
-	start = strtoul(argv[3], NULL, 10);
-	if (start == ULONG_MAX && errno == ERANGE)
+	errno = 0;
+	start = strtoul_noneg(argv[3], NULL, 10);
+	if (errno != 0)
 		exit(1);
-	count = strtoul(argv[4], NULL, 10);
-	if (count == ULONG_MAX && errno == ERANGE)
+	count = strtoul_noneg(argv[4], NULL, 10);
+	if (errno != 0)
 		exit(1);
 	if (check_uids) {
 		if (have_sub_uids(owner, start, count))
