@@ -22,12 +22,16 @@
 #ifdef HAVE_LL_HOST
 #include <net/if.h>
 #endif
+
 #include "defines.h"
 #include "prototypes.h"
 #include "getdef.h"
+#include "memzero.h"
 /*@-exitarg@*/
 #include "exitcodes.h"
 #include "shadowlog.h"
+#include "string/strftime.h"
+
 
 /*
  * Needed for MkLinux DR1/2/2.1 - J.
@@ -119,7 +123,7 @@ static void print_one (/*@null@*/const struct passwd *pw)
 		if (fread (&ll, sizeof (ll), 1, lastlogfile) != 1) {
 			fprintf (stderr,
 			         _("%s: Failed to get the entry for UID %lu\n"),
-			         Prog, (unsigned long int)pw->pw_uid);
+			         Prog, (unsigned long)pw->pw_uid);
 			exit (EXIT_FAILURE);
 		}
 	} else {
@@ -155,7 +159,7 @@ static void print_one (/*@null@*/const struct passwd *pw)
 	if (tm == NULL) {
 		cp = "(unknown)";
 	} else {
-		strftime (ptime, sizeof (ptime), "%a %b %e %H:%M:%S %z %Y", tm);
+		STRFTIME(ptime, "%a %b %e %H:%M:%S %z %Y", tm);
 		cp = ptime;
 	}
 	if (ll.ll_time == (time_t) 0) {
@@ -241,7 +245,7 @@ static void update_one (/*@null@*/const struct passwd *pw)
 	if (fwrite (&ll, sizeof(ll), 1, lastlogfile) != 1) {
 			fprintf (stderr,
 			         _("%s: Failed to update the entry for UID %lu\n"),
-			         Prog, (unsigned long int)pw->pw_uid);
+			         Prog, (unsigned long)pw->pw_uid);
 			exit (EXIT_FAILURE);
 	}
 }
@@ -322,7 +326,7 @@ int main (int argc, char **argv)
 			case 'b':
 			{
 				unsigned long inverse_days;
-				if (getulong (optarg, &inverse_days) == 0) {
+				if (getulong(optarg, &inverse_days) == -1) {
 					fprintf (stderr,
 					         _("%s: invalid numeric argument '%s'\n"),
 					         Prog, optarg);
@@ -350,7 +354,7 @@ int main (int argc, char **argv)
 			case 't':
 			{
 				unsigned long days;
-				if (getulong (optarg, &days) == 0) {
+				if (getulong(optarg, &days) == -1) {
 					fprintf (stderr,
 					         _("%s: invalid numeric argument '%s'\n"),
 					         Prog, optarg);
@@ -379,9 +383,9 @@ int main (int argc, char **argv)
 					umax = umin;
 					has_umax = true;
 				} else {
-					if (getrange (optarg,
-					              &umin, &has_umin,
-					              &umax, &has_umax) == 0) {
+					if (getrange(optarg,
+					             &umin, &has_umin,
+					             &umax, &has_umax) == -1) {
 						fprintf (stderr,
 						         _("%s: Unknown user or range: %s\n"),
 						         Prog, optarg);
